@@ -1,7 +1,7 @@
-/* $Id: eval.c,v 1.1.1.1 2007/01/11 15:49:52 dhartmei Exp $ */
+/* $Id: eval.c,v 1.3 2019/01/28 17:12:52 dhartmei Exp $ */
 
 /*
- * Copyright (c) 2004-2006 Daniel Hartmeier
+ * Copyright (c) 2004-2019 Daniel Hartmeier
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,7 +30,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: eval.c,v 1.1.1.1 2007/01/11 15:49:52 dhartmei Exp $";
+static const char rcsid[] = "$Id: eval.c,v 1.3 2019/01/28 17:12:52 dhartmei Exp $";
 
 #include <errno.h>
 #include <pthread.h>
@@ -43,7 +43,7 @@ static const char rcsid[] = "$Id: eval.c,v 1.1.1.1 2007/01/11 15:49:52 dhartmei
 #define REG_BASIC	0
 #endif
 
-extern int	 yyerror(char *, ...);
+extern int	 yyerror(const char *, ...);
 extern void	 die(const char *);
 static void	 mutex_lock(void);
 static void	 mutex_unlock(void);
@@ -326,11 +326,14 @@ void
 eval_clear(struct ruleset *rs, int *res, int type)
 {
 	struct cond_list *cl;
+	struct action_list *al;
 
 	mutex_lock();
 	for (; type < COND_MAX; ++type)
 		for (cl = rs->cond[type]; cl != NULL; cl = cl->next)
 			push_cond_result(cl->cond, VAL_UNDEF, res);
+	for (al = rs->action; al != NULL; al = al->next)
+		res[al->action->idx] = VAL_UNDEF;
 	mutex_unlock();
 }
 
@@ -364,7 +367,7 @@ push_expr_result(struct expr *e, int val, int *res)
 	if (res[e->idx] == val)
 		return;
 	res[e->idx] = val;
-	if (e->action != NULL)
+	if (e->action != NULL && val == VAL_TRUE)
 		res[e->action->idx] = val;
 	for (el = e->expr; el != NULL; el = el->next) {
 		struct expr *p = el->expr;
