@@ -1,7 +1,7 @@
-/* $Id: eval.h,v 1.7 2004/10/29 14:48:41 dhartmei Exp $ */
+/* $Id: eval.h,v 1.1.1.1 2007/01/11 15:49:52 dhartmei Exp $ */
 
 /*
- * Copyright (c) 2004 Daniel Hartmeier
+ * Copyright (c) 2004-2006 Daniel Hartmeier
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,7 @@
 #include <regex.h>
 
 enum { VAL_UNDEF=0, VAL_TRUE, VAL_FALSE };
-enum { COND_CONNECT, COND_HELO, COND_ENVFROM, COND_ENVRCPT,
+enum { COND_MACRO, COND_CONNECT, COND_HELO, COND_ENVFROM, COND_ENVRCPT,
     COND_HEADER, COND_BODY, COND_MAX };
 enum { EXPR_AND, EXPR_OR, EXPR_NOT, EXPR_COND };
 enum { ACTION_REJECT, ACTION_TEMPFAIL, ACTION_QUARANTINE,
@@ -103,7 +103,7 @@ struct expr	*create_expr(struct ruleset *, int, struct expr *,
 struct action	*create_action(struct ruleset *, int, const char *);
 struct action	*eval_cond(struct ruleset *, int *, int,
 		    const char *, const char *);
-struct action	*eval_end(struct ruleset *, int *, int);
+struct action	*eval_end(struct ruleset *, int *, int, int);
 void		 eval_clear(struct ruleset *, int *, int);
 void		 free_ruleset(struct ruleset *);
 
