@@ -1,7 +1,7 @@
-/* $Id: milter-regex.c,v 1.3 2007/08/03 22:11:48 dhartmei Exp $ */
+/* $Id: milter-regex.c,v 1.22 2019/12/12 14:43:01 dhartmei Exp $ */
 
 /*
- * Copyright (c) 2003-2006 Daniel Hartmeier
+ * Copyright (c) 2003-2019 Daniel Hartmeier
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,7 +30,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: milter-regex.c,v 1.3 2007/08/03 22:11:48 dhartmei Exp $";
+static const char rcsid[] = "$Id: milter-regex.c,v 1.22 2019/12/12 14:43:01 dhartmei Exp $";
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -39,16 +39,18 @@ static const char rcsid[] = "$Id: milter-regex.c,v 1.3 2007/08/03 22:11:48 dhart
 #include <arpa/inet.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <grp.h>
 #include <pthread.h>
 #include <pwd.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#define SYSLOG_NAMES
 #include <syslog.h>
 #include <unistd.h>
 #ifdef __linux__
-#include <grp.h>
+#include <stdbool.h>
 #endif
 #include <libmilter/mfapi.h>
 
@@ -60,6 +62,7 @@ extern int	 parse_ruleset(const char *, struct ruleset **, char *,
 
 static const char	*rule_file_name = "/etc/milter-regex.conf";
 static int		 debug = 0;
+static unsigned		 maxlines = 0;
 static pthread_mutex_t	 mutex;
 
 struct context {
@@ -70,10 +73,13 @@ struct context {
 	char		 host_name[128];
 	char		 host_addr[64];
 	char		 helo[128];
+	char		 env_from[128];
+	char		 env_rcpt[2048];
 	char		 hdr_from[128];
 	char		 hdr_to[128];
 	char		 hdr_subject[128];
 	char		*quarantine;
+	unsigned	 lines;
 };
 
 static sfsistat		 setreply(SMFICTX *, struct context *,
@@ -117,7 +123,14 @@ struct {
 	{ "helo", "{cipher_bits}" },
 	{ "helo", "{cert_subject}" },
 	{ "helo", "{cert_issuer}" },
+	{ "helo", "{verify}" },
 	{ "envfrom", "i" },
+	{ "envfrom", "{tls_version}" },
+	{ "envfrom", "{cipher}" },
+	{ "envfrom", "{cipher_bits}" },
+	{ "envfrom", "{cert_subject}" },
+	{ "envfrom", "{cert_issuer}" },
+	{ "envfrom", "{verify}" },
 	{ "envfrom", "{auth_type}" },
 	{ "envfrom", "{auth_authen}" },
 	{ "envfrom", "{auth_ssf}" },
@@ -133,6 +146,7 @@ struct {
 
 #if __linux__ || __sun__
 #define	ST_MTIME st_mtime
+extern size_t	 strlcat(char *, const char *, size_t);
 extern size_t	 strlcpy(char *, const char *, size_t);
 #else
 #define	ST_MTIME st_mtimespec
@@ -160,21 +174,21 @@ daemon(int nochdir, int noclose)
 	int fd;
 
 	if ((pid = fork()) < 0) {
-		fprintf(stderr, "fork: %s\n", strerror(errno));
+		perror("fork");
 		return (1);
 	} else if (pid > 0)
 		_exit(0);
 	if ((pid = setsid()) == -1) {
-		fprintf(stderr, "setsid: %s\n", strerror(errno));
+		perror("setsid");
 		return (1);
 	}
 	if ((pid = fork()) < 0) {
-		fprintf(stderr, "fork: %s\n", strerror(errno));
+		perror("fork");
 		return (1);
 	} else if (pid > 0)
 		_exit(0);
 	if (!nochdir && chdir("/")) {
-		fprintf(stderr, "chdir: %s\n", strerror(errno));
+		perror("chdir");
 		return (1);
 	}
 	if (!noclose) {
@@ -193,14 +207,16 @@ setreply(SMFICTX *ctx, struct context *context, const struct action *action)
 
 	switch (action->type) {
 	case ACTION_REJECT:
-		msg(LOG_NOTICE, context, "REJECT: %s, Helo: %s, From: %s, "
-		    "To: %s, Subject: %s", action->msg, context->helo,
+		msg(LOG_NOTICE, context, "REJECT: %s, HELO: %s, FROM: %s, "
+		    "RCPT: %s, From: %s, To: %s, Subject: %s", action->msg,
+		    context->helo, context->env_from, context->env_rcpt,
 		    context->hdr_from, context->hdr_to, context->hdr_subject);
 		result = SMFIS_REJECT;
 		break;
 	case ACTION_TEMPFAIL:
-		msg(LOG_NOTICE, context, "TEMPFAIL: %s, Helo: %s, From: %s, "
-		    "To: %s, Subject: %s", action->msg, context->helo,
+		msg(LOG_NOTICE, context, "TEMPFAIL: %s, HELO: %s, FROM: %s, "
+		    "RCPT: %s, From: %s, To: %s, Subject: %s", action->msg,
+		    context->helo, context->env_from, context->env_rcpt,
 		    context->hdr_from, context->hdr_to, context->hdr_subject);
 		result = SMFIS_TEMPFAIL;
 		break;
@@ -210,15 +226,17 @@ setreply(SMFICTX *ctx, struct context *context, const struct action *action)
 		context->quarantine = strdup(action->msg);
 		break;
 	case ACTION_DISCARD:
-		msg(LOG_NOTICE, context, "DISCARD, Helo: %s, From: %s, "
-		    "To: %s, Subject: %s", context->helo, context->hdr_from,
-		    context->hdr_to, context->hdr_subject);
+		msg(LOG_NOTICE, context, "DISCARD, HELO: %s, FROM: %s, "
+		    "RCPT: %s, From: %s, To: %s, Subject: %s",
+		    context->helo, context->env_from, context->env_rcpt,
+		    context->hdr_from, context->hdr_to, context->hdr_subject);
 		result = SMFIS_DISCARD;
 		break;
 	case ACTION_ACCEPT:
-		msg(LOG_INFO, context, "ACCEPT, Helo: %s, From: %s, "
-		    "To: %s, Subject: %s", context->helo, context->hdr_from,
-		    context->hdr_to, context->hdr_subject);
+		msg(LOG_INFO, context, "ACCEPT, HELO: %s, FROM: %s, "
+		    "RCPT: %s, From: %s, To: %s, Subject: %s",
+		    context->helo, context->env_from, context->env_rcpt,
+		    context->hdr_from, context->hdr_to, context->hdr_subject);
 		result = SMFIS_ACCEPT;
 		break;
 	}
@@ -427,6 +445,7 @@ cb_envfrom(SMFICTX *ctx, char **args)
 	eval_clear(context->rs, context->res, COND_ENVFROM);
 	if (*args != NULL) {
 		msg(LOG_DEBUG, context, "cb_envfrom('%s')", *args);
+		strlcpy(context->env_from, *args, sizeof(context->env_from));
 		if ((action = eval_cond(context->rs, context->res, COND_ENVFROM,
 		    *args, NULL)) != NULL)
 			return (setreply(ctx, context, action));
@@ -453,6 +472,10 @@ cb_envrcpt(SMFICTX *ctx, char **args)
 	eval_clear(context->rs, context->res, COND_ENVRCPT);
 	if (*args != NULL) {
 		msg(LOG_DEBUG, context, "cb_envrcpt('%s')", *args);
+		if (context->env_rcpt[0])
+			strlcat(context->env_rcpt, " ",
+			    sizeof(context->env_rcpt));
+		strlcat(context->env_rcpt, *args, sizeof(context->env_rcpt));
 		if ((action = eval_cond(context->rs, context->res, COND_ENVRCPT,
 		    *args, NULL)) != NULL)
 			return (setreply(ctx, context, action));
@@ -532,6 +555,8 @@ cb_body(SMFICTX *ctx, u_char *chunk, size_t size)
 			else
 				context->buf[context->pos] = 0;
 			context->pos = 0;
+			if (maxlines && context->lines++ > maxlines)
+				continue;
 			msg(LOG_DEBUG, context, "cb_body('%s')", context->buf);
 			if ((action = eval_cond(context->rs, context->res,
 			    COND_BODY, context->buf, NULL)) != NULL)
@@ -558,16 +583,24 @@ cb_eom(SMFICTX *ctx)
 	    COND_MAX)) != NULL)
 		result = setreply(ctx, context, action);
 	else
-		msg(LOG_DEBUG, context, "ACCEPT, Helo: %s, From: %s, To: %s, "
-		    "Subject: %s", context->helo, context->hdr_from,
-		    context->hdr_to, context->hdr_subject);
+		msg(LOG_DEBUG, context, "ACCEPT, HELO: %s, FROM: %s, "
+		    "RCPT: %s, From: %s, To: %s, Subject: %s",
+		    context->helo, context->env_from, context->env_rcpt,
+		    context->hdr_from, context->hdr_to, context->hdr_subject);
 	if (context->quarantine != NULL) {
-		msg(LOG_NOTICE, context, "QUARANTINE: %s, Helo: %s, From: %s, "
-		    "To: %s, Subject: %s", action->msg, context->helo,
+		msg(LOG_NOTICE, context, "QUARANTINE: %s, HELO: %s, FROM: %s, "
+		    "RCPT: %s, From: %s, To: %s, Subject: %s", action->msg,
+		    context->helo, context->env_from, context->env_rcpt,
 		    context->hdr_from, context->hdr_to, context->hdr_subject);
 		if (smfi_quarantine(ctx, context->quarantine) != MI_SUCCESS)
 			msg(LOG_ERR, context, "cb_eom: smfi_quarantine");
 	}
+	context->pos = context->hdr_from[0] = context->hdr_to[0] =
+	    context->hdr_subject[0] = 0;
+	if (context->quarantine != NULL) {
+		free(context->quarantine);
+		context->quarantine = NULL;
+	}
 	return (result);
 }
 
@@ -602,7 +635,10 @@ struct smfiDesc smfilter = {
 	cb_body,	/* body block */
 	cb_eom,		/* end of message */
 	NULL,		/* message aborted */
-	cb_close	/* connection cleanup */
+	cb_close,	/* connection cleanup */
+	NULL,		/* unrecognized or unimplemented command filter */
+	NULL,		/* SMTP DATA command filter */
+	NULL		/* negotiation callback */
 };
 
 static void
@@ -613,7 +649,8 @@ msg(int priority, struct context *context, const char *fmt, ...)
 
 	va_start(ap, fmt);
 	if (context != NULL)
-		snprintf(msg, sizeof(msg), "%s: ", context->host_addr);
+		snprintf(msg, sizeof(msg), "%s [%s]: ", context->host_name,
+		    context->host_addr);
 	else
 		msg[0] = 0;
 	vsnprintf(msg + strlen(msg), sizeof(msg) - strlen(msg), fmt, ap);
@@ -627,8 +664,9 @@ msg(int priority, struct context *context, const char *fmt, ...)
 static void
 usage(const char *argv0)
 {
-	fprintf(stderr, "usage: %s [-d] [-c config] [-u user] "
-	    "[-p pipe]\n", argv0);
+	fprintf(stderr, "usage: %s [-dt] [-c config] [-f facility] "
+	    "[-j dirname] [-l loglevel] [-m number] [-p pipe] [-r pid-file] "
+	    "[-u user] [-G group] [-P mode] [-U user]\n", argv0);
 	exit(1);
 }
 
@@ -645,16 +683,21 @@ die(const char *reason)
 int
 main(int argc, char **argv)
 {
-	int ch;
+	int ch, maskpri = LOG_INFO;
 	const char *oconn = OCONN;
+	const char *pid_file_name = NULL;
 	const char *user = USER;
+	const char *jail = NULL;
 	sfsistat r = MI_FAILURE;
 	const char *ofile = NULL;
+	const char *pgroup = NULL;
+	const char *puser = NULL;
+	mode_t pperm = 0600;
+	int facility = LOG_DAEMON;
+	int test_ruleset = 0;
+	FILE *f = NULL;
 
-	tzset();
-	openlog("milter-regex", LOG_PID | LOG_NDELAY, LOG_DAEMON);
-
-	while ((ch = getopt(argc, argv, "c:dp:u:")) != -1) {
+	while ((ch = getopt(argc, argv, "c:df:j:l:m:p:r:tu:G:P:U:")) != -1) {
 		switch (ch) {
 		case 'c':
 			rule_file_name = optarg;
@@ -662,12 +705,46 @@ main(int argc, char **argv)
 		case 'd':
 			debug = 1;
 			break;
+		case 'f': {
+			int i;
+
+			for (i = 0; facilitynames[i].c_name != NULL; ++i)
+				if (!strcmp(facilitynames[i].c_name, optarg)) {
+					facility = facilitynames[i].c_val;
+					break;
+				}
+			break;
+		}
+		case 'j':
+			jail = optarg;
+			break;
+		case 'l':
+			maskpri = atoi(optarg);
+			break;
+		case 'm':
+			maxlines = atoi(optarg);
+			break;
 		case 'p':
 			oconn = optarg;
 			break;
+		case 't':
+			test_ruleset = 1;
+			break;
+		case 'r':
+			pid_file_name = optarg;
+			break;
 		case 'u':
 			user = optarg;
 			break;
+		case 'G':
+			pgroup = optarg;
+			break;
+		case 'P':
+			pperm = strtol(optarg, (char **)NULL, 8);
+			break;
+		case 'U':
+			puser = optarg;
+			break;
 		default:
 			usage(argv[0]);
 		}
@@ -678,40 +755,24 @@ main(int argc, char **argv)
 		usage(argv[0]);
 	}
 
+	tzset();
+	openlog("milter-regex", LOG_PID | LOG_NDELAY, facility);
+	setlogmask(LOG_UPTO(maskpri));
+
 	if (!strncmp(oconn, "unix:", 5))
 		ofile = oconn + 5;
 	else if (!strncmp(oconn, "local:", 6))
 		ofile = oconn + 6;
-	if (ofile != NULL)
-		unlink(ofile);
+	else if (!strchr(oconn, ':'))
+		ofile = oconn;
 
-	/* drop privileges */
-	if (!getuid()) {
-		struct passwd *pw;
-
-		if ((pw = getpwnam(user)) == NULL) {
-			fprintf(stderr, "getpwnam: %s: %s\n", user,
-			    strerror(errno));
-			return (1);
-		}
-		setgroups(1, &pw->pw_gid);
-		if (setegid(pw->pw_gid) || setgid(pw->pw_gid)) {
-			fprintf(stderr, "setgid: %s\n", strerror(errno));
-			return (1);
-		}
-		if (
-#if ! ( __linux__ || __sun__ )
-		    seteuid(pw->pw_uid) ||
-#endif
-		    setuid(pw->pw_uid)) {
-			fprintf(stderr, "setuid: %s\n", strerror(errno));
-			return (1);
-		}
+	if (jail != NULL && (chroot(jail) || chdir("/"))) {
+		perror("chroot");
+		return (1);
 	}
 
-	if (pthread_mutex_init(&mutex, 0)) {
-		fprintf(stderr, "pthread_mutex_init\n");
-		goto done;
+	if (test_ruleset) {
+		return (get_ruleset() == NULL ? 1 : 0);
 	}
 
 	if (smfi_setconn((char *)oconn) != MI_SUCCESS) {
@@ -724,6 +785,78 @@ main(int argc, char **argv)
 		goto done;
 	}
 
+	if (pid_file_name != NULL) {
+		umask(0133);
+		if ((f = fopen(pid_file_name, "w")) == NULL) {
+			fprintf(stderr, "fopen: %s: %s\n", pid_file_name,
+			    strerror(errno));
+			return (1);
+		}
+	}
+
+	umask(0777);
+	if (smfi_opensocket(true) != MI_SUCCESS) {
+		fprintf(stderr, "smfi_opensocket: failed\n");
+		goto done;
+	}
+
+	/* modify socket file ownership and permissions, drop privileges */
+	if (!getuid()) {
+		struct passwd *pw;
+		struct group *gr;
+		uid_t uid = -1;
+		gid_t gid = -1;
+
+		if (ofile != NULL && puser != NULL) {
+			if ((pw = getpwnam(puser)) == NULL) {
+				fprintf(stderr, "getpwnam: %s: %s\n", puser,
+				    strerror(errno));
+				return (1);
+			}
+			uid = pw->pw_uid;
+		}
+		if (ofile != NULL && pgroup != NULL) {
+			if ((gr = getgrnam(pgroup)) == NULL) {
+				fprintf(stderr, "getgrnam: %s: %s\n", pgroup,
+				    strerror(errno));
+				return (1);
+			}
+			gid = gr->gr_gid;
+		}
+		if ((uid != -1 || gid != -1) && chown(ofile, uid, gid)) {
+			fprintf(stderr, "chown: %s: %s\n", ofile,
+			    strerror(errno));
+			return (1);
+		}
+		if (ofile != NULL && chmod(ofile, pperm)) {
+			fprintf(stderr, "chmod: %s: %s\n", ofile,
+			    strerror(errno));
+			return (1);
+		}
+		if ((pw = getpwnam(user)) == NULL) {
+			fprintf(stderr, "getpwnam: %s: %s\n", user,
+			    strerror(errno));
+			return (1);
+		}
+		if (setgroups(1, &pw->pw_gid)) {
+			perror("setgroups");
+			return (1);
+		}
+		if (setgid(pw->pw_gid)) {
+			perror("setgid");
+			return (1);
+		}
+		if (setuid(pw->pw_uid)) {
+			perror("setuid");
+			return (1);
+		}
+	}
+
+	if (pthread_mutex_init(&mutex, 0)) {
+		fprintf(stderr, "pthread_mutex_init\n");
+		goto done;
+	}
+
 	if (eval_init(ACTION_ACCEPT)) {
 		fprintf(stderr, "eval_init: failed\n");
 		goto done;
@@ -731,10 +864,14 @@ main(int argc, char **argv)
 
 	/* daemonize (detach from controlling terminal) */
 	if (!debug && daemon(0, 0)) {
-		fprintf(stderr, "daemon: %s\n", strerror(errno));
+		perror("daemon");
 		goto done;
 	}
-	umask(0177);
+
+	if (f != NULL) {
+		fprintf(f, "%d", (int)getpid());
+		fclose(f);
+	}
 
 	msg(LOG_INFO, NULL, "started: %s", rcsid);
 	r = smfi_main();
